@@ -7,7 +7,7 @@ import unittest
 import deeptrack.scatterers as scatterers
 
 import numpy as np
-from deeptrack.optics import Fluorescence
+from deeptrack.optics import Fluorescence, Brightfield
 from deeptrack.image import Image
 
 
@@ -92,6 +92,23 @@ class TestScatterers(unittest.TestCase):
         self.assertIsInstance(output_image, Image)
         self.assertEqual(output_image.shape, (64, 64, 1))
 
+    def test_MieSphere(self):
+        optics_1 = Brightfield(
+            NA=0.7,
+            wavelength=680e-9,
+            resolution=1e-6,
+            magnification=5,
+            output_region=(0, 0, 64, 64),
+            padding=(128, 128, 128, 128),
+            return_field=True,
+            upscale=2,
+        )
+        scatterer = scatterers.MieSphere(radius=1e-6, refractive_index=1.45 + 0.1j)
+
+        imaged_scatterer_1 = optics_1(scatterer)
+
+        image_1 = imaged_scatterer_1.resolve()
+
 
 if __name__ == "__main__":
     unittest.main()
\ No newline at end of file
